@@ -252,23 +252,14 @@ lyplg_type_validate_instanceid(const struct ly_ctx *ctx, const struct lysc_type
     return LY_SUCCESS;
 }
 
-LIBYANG_API_DEF LY_ERR
-lyplg_type_compare_instanceid(const struct ly_ctx *ctx, const struct lyd_value *val1, const struct lyd_value *val2)
-{
-    return lyplg_type_compare_simple(ctx, val1, val2);
-}
-
-LIBYANG_API_DEF int
-lyplg_type_sort_instanceid(const struct ly_ctx *ctx, const struct lyd_value *val1, const struct lyd_value *val2)
-{
-    return lyplg_type_sort_simple(ctx, val1, val2);
-}
-
 LIBYANG_API_DEF const void *
-lyplg_type_print_instanceid(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *value, LY_VALUE_FORMAT format,
+lyplg_type_print_instanceid(const struct ly_ctx *ctx, const struct lyd_value *value, LY_VALUE_FORMAT format,
         void *prefix_data, ly_bool *dynamic, size_t *value_len)
 {
-    char *ret;
+    char *ret = NULL;
+    struct ly_path *p = NULL;
+    const struct ly_path *target;
+    struct ly_err_item *err;
 
     if ((format == LY_VALUE_CANON) || (format == LY_VALUE_JSON) || (format == LY_VALUE_LYB)) {
         if (dynamic) {
@@ -280,14 +271,34 @@ lyplg_type_print_instanceid(const struct ly_ctx *UNUSED(ctx), const struct lyd_v
         return value->_canonical;
     }
 
-    /* print the value in the specific format */
-    if (instanceid_path2str(value->target, format, prefix_data, &ret)) {
-        return NULL;
+    if (!value->target) {
+        /* schema default value, compile it first */
+        if (lyplg_type_lypath_new(ctx, value->_canonical, strlen(value->_canonical), 0, LY_VALUE_JSON, NULL, NULL,
+                NULL, &p, &err)) {
+            if (err) {
+                ly_err_print(ctx, err);
+                ly_err_free(err);
+            }
+            goto cleanup;
+        }
+
+        target = p;
+    } else {
+        target = value->target;
     }
+
+    /* print the value in the specific format */
+    if (instanceid_path2str(target, format, prefix_data, &ret)) {
+        goto cleanup;
+    }
+
     *dynamic = 1;
     if (value_len) {
         *value_len = strlen(ret);
     }
+
+cleanup:
+    ly_path_free(p);
     return ret;
 }
 
@@ -295,6 +306,7 @@ LIBYANG_API_DEF LY_ERR
 lyplg_type_dup_instanceid(const struct ly_ctx *ctx, const struct lyd_value *original, struct lyd_value *dup)
 {
     LY_ERR ret;
+    struct ly_err_item *err;
 
     memset(dup, 0, sizeof *dup);
 
@@ -302,9 +314,21 @@ lyplg_type_dup_instanceid(const struct ly_ctx *ctx, const struct lyd_value *orig
     ret = lydict_insert(ctx, original->_canonical, 0, &dup->_canonical);
     LY_CHECK_GOTO(ret, error);
 
-    /* copy path */
-    ret = ly_path_dup(ctx, original->target, &dup->target);
-    LY_CHECK_GOTO(ret, error);
+    if (!original->target) {
+        /* schema default value, needs to be compiled */
+        if (lyplg_type_lypath_new(ctx, original->_canonical, strlen(original->_canonical), 0, LY_VALUE_JSON, NULL, NULL,
+                NULL, &dup->target, &err)) {
+            if (err) {
+                ly_err_print(ctx, err);
+                ly_err_free(err);
+            }
+            goto error;
+        }
+    } else {
+        /* copy path */
+        ret = ly_path_dup(ctx, original->target, &dup->target);
+        LY_CHECK_GOTO(ret, error);
+    }
 
     dup->realtype = original->realtype;
     return LY_SUCCESS;
@@ -319,7 +343,7 @@ lyplg_type_free_instanceid(const struct ly_ctx *ctx, struct lyd_value *value)
 {
     lydict_remove(ctx, value->_canonical);
     value->_canonical = NULL;
-    ly_path_free(ctx, value->target);
+    ly_path_free(value->target);
 }
 
 /**
@@ -338,8 +362,8 @@ const struct lyplg_type_record plugins_instanceid[] = {
         .plugin.id = "libyang 2 - instance-identifier, version 1",
         .plugin.store = lyplg_type_store_instanceid,
         .plugin.validate = lyplg_type_validate_instanceid,
-        .plugin.compare = lyplg_type_compare_instanceid,
-        .plugin.sort = lyplg_type_sort_instanceid,
+        .plugin.compare = lyplg_type_compare_simple,
+        .plugin.sort = lyplg_type_sort_simple,
         .plugin.print = lyplg_type_print_instanceid,
         .plugin.duplicate = lyplg_type_dup_instanceid,
         .plugin.free = lyplg_type_free_instanceid,
