@@ -168,7 +168,7 @@ lys_getnext_(const struct lysc_node *last, const struct lysc_node *parent, const
     const struct lysc_node *next = NULL;
     ly_bool action_flag = 0, notif_flag = 0, sm_flag = options & LYS_GETNEXT_WITHSCHEMAMOUNT ? 0 : 1;
     LY_ARRAY_COUNT_TYPE u;
-    struct ly_ctx *sm_ctx = NULL;
+    const struct ly_ctx *sm_ctx = NULL;
     const struct lys_module *mod;
     uint32_t idx;
 
@@ -259,7 +259,7 @@ repeat:
                 LY_ARRAY_FOR(parent->exts, u) {
                     if (!strcmp(parent->exts[u].def->name, "mount-point") &&
                             !strcmp(parent->exts[u].def->module->name, "ietf-yang-schema-mount")) {
-                        lyplg_ext_schema_mount_create_context(&parent->exts[u], &sm_ctx);
+                        lyplg_ext_schema_mount_get_ctx(&parent->exts[u], &sm_ctx);
                         if (sm_ctx) {
                             /* some usable context created */
                             break;
@@ -276,10 +276,6 @@ repeat:
 
                         next = lys_getnext(NULL, NULL, mod->compiled, options & ~LYS_GETNEXT_WITHSCHEMAMOUNT);
                     }
-                    if (!next) {
-                        /* no nodes found */
-                        ly_ctx_destroy(sm_ctx);
-                    }
                 }
             }
         } else {
@@ -846,7 +842,7 @@ _lys_set_implemented(struct lys_module *mod, const char **features, struct lys_g
         }
     }
 
-    /* Try to find module with LYS_MOD_IMPORTED_REV flag. */
+    /* try to find module with LYS_MOD_IMPORTED_REV flag */
     i = 0;
     while ((mod_iter = ly_ctx_get_module_iter(mod->ctx, &i))) {
         if (!strcmp(mod_iter->name, mod->name) && (mod_iter != mod) && (mod_iter->latest_revision & LYS_MOD_IMPORTED_REV)) {
@@ -1198,6 +1194,9 @@ lys_set_implemented(struct lys_module *mod, const char **features)
         lys_unres_glob_erase(unres);
     }
 
+    /* new context state */
+    ly_ctx_new_change(mod->ctx);
+
 cleanup:
     if (ret) {
         lys_unres_glob_revert(mod->ctx, unres);
@@ -1492,7 +1491,7 @@ lysp_load_module_data_check(const struct ly_ctx *ctx, struct lysp_module *mod, s
     latest_revision = mod ? mod->mod->latest_revision : submod->latest_revision;
 
     if (mod_data->name) {
-        /* check name of the parsed model */
+        /* check name of the parsed module */
         if (strcmp(mod_data->name, name)) {
             LOGERR(ctx, LY_EINVAL, "Unexpected module \"%s\" parsed instead of \"%s\".", name, mod_data->name);
             return LY_EINVAL;
@@ -1500,7 +1499,7 @@ lysp_load_module_data_check(const struct ly_ctx *ctx, struct lysp_module *mod, s
     }
 
     if (mod_data->revision) {
-        /* check revision of the parsed model */
+        /* check revision of the parsed module */
         if (!revs || strcmp(mod_data->revision, revs[0].date)) {
             LOGERR(ctx, LY_EINVAL, "Module \"%s\" parsed with the wrong revision (\"%s\" instead \"%s\").", name,
                     revs ? revs[0].date : "none", mod_data->revision);
@@ -1536,7 +1535,8 @@ lysp_load_module_data_check(const struct ly_ctx *ctx, struct lysp_module *mod, s
 
 LY_ERR
 lys_parse_submodule(struct ly_ctx *ctx, struct ly_in *in, LYS_INFORMAT format, struct lysp_ctx *main_ctx,
-        const struct lysp_load_module_data *mod_data, struct ly_set *new_mods, struct lysp_submodule **submodule)
+        const struct lysp_load_module_data *mod_data, ly_bool in_searchdirs, struct ly_set *new_mods,
+        struct lysp_submodule **submodule)
 {
     LY_ERR rc = LY_SUCCESS, r;
     struct lysp_submodule *submod = NULL, *latest_sp;
@@ -1544,10 +1544,11 @@ lys_parse_submodule(struct ly_ctx *ctx, struct ly_in *in, LYS_INFORMAT format, s
     struct lysp_yin_ctx *yinctx = NULL;
     struct lysp_ctx *pctx = NULL;
     struct lysf_ctx fctx = {.ctx = ctx};
-    const char *submod_name;
 
     LY_CHECK_ARG_RET(ctx, ctx, in, LY_EINVAL);
 
+    *submodule = NULL;
+
     switch (format) {
     case LYS_IN_YIN:
         rc = yin_parse_submodule(&yinctx, ctx, main_ctx, in, &submod);
@@ -1586,24 +1587,24 @@ lys_parse_submodule(struct ly_ctx *ctx, struct ly_in *in, LYS_INFORMAT format, s
             latest_sp = NULL;
         }
     } else {
-        submod->latest_revision = 1;
+        /* if found in searchdirs and looking for the latest revision, it is the latest revision available,
+         * otherwise the only such submodule in the context */
+        submod->latest_revision = (in_searchdirs && !mod_data->revision) ? 2 : 1;
     }
 
-    if (mod_data) {
-        /* check the parsed submodule it is as expected */
-        r = lysp_load_module_data_check(ctx, NULL, submod, mod_data);
-        if (r == LY_EEXIST) {
-            /* not an error, the submodule already exists so free this one */
-            ly_set_erase(&pctx->tpdfs_nodes, NULL);
-            ly_set_erase(&pctx->grps_nodes, NULL);
-            ly_set_erase(&pctx->ext_inst, NULL);
-            lysp_module_free(&fctx, (struct lysp_module *)submod);
-            submod = NULL;
-            goto cleanup;
-        } else if (r) {
-            rc = r;
-            goto cleanup;
-        }
+    /* check the parsed submodule is as expected */
+    r = lysp_load_module_data_check(ctx, NULL, submod, mod_data);
+    if (r == LY_EEXIST) {
+        /* not an error, the submodule already exists so free this one */
+        ly_set_erase(&pctx->tpdfs_nodes, NULL);
+        ly_set_erase(&pctx->grps_nodes, NULL);
+        ly_set_erase(&pctx->ext_inst, NULL);
+        lysp_module_free(&fctx, (struct lysp_module *)submod);
+        submod = NULL;
+        goto cleanup;
+    } else if (r) {
+        rc = r;
+        goto cleanup;
     }
 
     if (latest_sp) {
@@ -1617,19 +1618,7 @@ lys_parse_submodule(struct ly_ctx *ctx, struct ly_in *in, LYS_INFORMAT format, s
 
 cleanup:
     if (rc) {
-        if (submod && submod->name) {
-            submod_name = submod->name;
-        } else if (mod_data) {
-            submod_name = mod_data->name;
-        } else {
-            submod_name = NULL;
-        }
-        if (submod_name) {
-            LOGERR(ctx, rc, "Parsing submodule \"%s\" failed.", submod_name);
-        } else {
-            LOGERR(ctx, rc, "Parsing submodule failed.");
-        }
-
+        LOGERR(ctx, rc, "Parsing submodule \"%s\" failed.", mod_data->name);
         if (pctx) {
             ly_set_erase(&pctx->tpdfs_nodes, NULL);
             ly_set_erase(&pctx->grps_nodes, NULL);
@@ -2190,7 +2179,6 @@ lys_parse_in(struct ly_ctx *ctx, struct ly_in *in, LYS_INFORMAT format, const st
     /* add into context */
     rc = ly_set_add(&ctx->list, mod, 1, NULL);
     LY_CHECK_GOTO(rc, cleanup);
-    ctx->change_count++;
 
     /* resolve includes and all imports */
     LY_CHECK_GOTO(rc = lysp_resolve_import_include(pctx, mod->parsed, new_mods), cleanup);
@@ -2299,6 +2287,9 @@ lys_parse(struct ly_ctx *ctx, struct ly_in *in, LYS_INFORMAT format, const char
         lys_unres_glob_erase(&ctx->unres);
     }
 
+    /* new context state */
+    ly_ctx_new_change(ctx);
+
 cleanup:
     if (ret) {
         lys_unres_glob_revert(ctx, &ctx->unres);
