@@ -825,14 +825,7 @@ lysc_ident_free(struct lysf_ctx *ctx, struct lysc_ident *ident)
     FREE_ARRAY(ctx, ident->exts, lysc_ext_instance_free);
 }
 
-/**
- * @brief Free the compiled range structure.
- *
- * @param[in] ctx Free context.
- * @param[in,out] range Compiled range structure to be freed.
- * Since the structure is typically part of the sized array, the structure itself is not freed.
- */
-static void
+void
 lysc_range_free(struct lysf_ctx *ctx, struct lysc_range *range)
 {
     LY_ARRAY_FREE(range->parts);
