@@ -375,12 +375,9 @@ test_data_xml(void **state)
     TEST_SUCCESS_XML("TPATTERN_0", "AHOJ", STRING, "AHOJ");
     /* test print error */
     TEST_ERROR_XML("TPATTERN_0", "T128");
-    CHECK_LOG_CTX("pattern 0 error message",
-            "Schema location \"/TPATTERN_0:port\", line number 1.");
+    CHECK_LOG_CTX("pattern 0 error message", "/TPATTERN_0:port", 1);
     TEST_ERROR_XML("TPATTERN_0", "ahoj");
-    CHECK_LOG_CTX("pattern 1 error message",
-            "Schema location \"/TPATTERN_0:port\", line number 1.");
-
+    CHECK_LOG_CTX("pattern 1 error message", "/TPATTERN_0:port", 1);
 }
 
 int
